
import base64
from urllib.parse import urlsplit, urlunsplit

from .exceptions import EmissionBadParamater
from .specifications import TORRENT_ARGUMENTS


########################
##### HTTP HELPERS #####
########################


def to_http(url):
    """
    change url's scheme
        >>> url = 'https://www.google.fr'
        >>> url = to_http(url)
        >>> http://www.google.fr
    """
    t_url = list(urlsplit(url))
    if t_url[0] == 'http' or t_url[0] == 'file':
        return url

    t_url[0] = 'http'
    new_url = urlunsplit(t_url)
    return new_url


def decoder(content):
    """
    All text MUST be UTF-8 encoded
    """
    torrent_data = base64.b64encode(content).decode('utf-8')
    return torrent_data


###########################
##### TORRENT HELPERS #####
###########################

def check_ids(ids):

    if not (isinstance(ids, int) or isinstance(ids, list) or isinstance(ids, str)):
        raise EmissionBadParamater('ids shoul be a int, list or string')

    if isinstance(ids, list):
        for v in ids:
            if isinstance(v, float):
                raise EmissionBadParamater('ids shoul not be a float')

    return ids


def to_dash(s):
    """
        >>> to_dash('delete_local_data')
        >>> delete-local-data
    """
    return s.replace('_', '-')


def from_dash(s):
    """
        >>> from_dash('delete-local-data')
        >>> delete_local_data
    """
    return s.replace('-', '_')


def check_method(method):

    if method not in TORRENT_ARGUMENTS.keys():
        raise EmissionBadParamater("'{0}' method does not exist".format(method))


def get_arguments_from_method(method):

    return TORRENT_ARGUMENTS[method]


def check_fields(method, args):

    def check_name(name, arguments):
        if name not in arguments.keys():
            raise EmissionBadParamater("'{0}' is not a valid argument".format(name))

    specs = get_arguments_from_method(method)

    for elt in args:
        check_name(elt, specs)


def check_arguments(method, args):

    def check_name(name, arguments):
        if name not in arguments.keys():
            raise EmissionBadParamater("'{0}' is not a valid argument".format(name))

    def check_value(name, value, arguments):
        if not isinstance(value, arguments[name][0]):
            raise EmissionBadParamater("'{0}' is not a valid value".format(value))

    specs = get_arguments_from_method(method)

    for elt in args:
        check_name(elt[0], specs)
        check_value(elt[0], elt[1], specs)

###############################################


def normalize_fields_arguments(method, args):

    check_method(method)

    # change '_' to '-'
    fields = [to_dash(k) for k in args]

    check_fields(method, fields)

    return fields


def normalize_arguments(method, kwargs):

    check_method(method)

    # change '_' to '-'
    options = [(to_dash(k), v) for k, v in kwargs.items()]

    # verify if the argument is valid
    check_arguments(method, options)

    return options




TORRENT_ARGUMENTS = {
    'torrent-start': {
        'ids': [
            (int, None, 'Request', 'an integer referring to a torrent id'),
            (list, None, 'Request', 'a list of torrent id numbers, sha1 hash strings, or both'),
            (str, None, 'Request', 'a string, "recently-active", for recently-active torrents')
        ]
    },
    'torrent-start-now': {
        'ids': [
            (int, None, 'Request', 'an integer referring to a torrent id'),
            (list, None, 'Request', 'a list of torrent id numbers, sha1 hash strings, or both'),
            (str, None, 'Request', 'a string, "recently-active", for recently-active torrents')
        ]
    },
    'torrent-stop': {
        'ids': [
            (int,  None, 'Request', 'an integer referring to a torrent id'),
            (list, None, 'Request', 'a list of torrent id numbers, sha1 hash strings, or both'),
            (str,  None, 'Request', 'a string, "recently-active", for recently-active torrents')
        ]
    },
    'torrent-verify': {
        'ids': [
            (int,  None, 'Request', 'an integer referring to a torrent id'),
            (list, None, 'Request', 'a list of torrent id numbers, sha1 hash strings, or both'),
            (str,  None, 'Request', 'a string, "recently-active", for recently-active torrents')
        ]
    },
    'torrent-reannounce': {
        'ids': [
            (int,  None, 'Request', 'an integer referring to a torrent id'),
            (list, None, 'Request', 'a list of torrent id numbers, sha1 hash strings, or both'),
            (str,  None, 'Request', 'a string, "recently-active", for recently-active torrents')
        ]
    },
    'torrent-set': {
        'bandwidthPriority':   (int,  None, 'Request', 'this torrents bandwidth tr_priority_t'),
        "downloadLimit":       (int,  None, 'Request', 'maximum download speed (KBps)'),
        "downloadLimited":     (bool, None, 'Request', 'true if "downloadLimit" is honored'),
        "files-wanted":        (list, None, 'Request', 'indices of file(s) to download'),
        "files-unwanted":      (list, None, 'Request', 'indices of file(s) to not download'),
        "honorsSessionLimits": (bool, None, 'Request', 'true if session upload limits are honored'),
        "ids":                 (list, None, 'Request', 'torrent list, as described in 3.1'),
        "location":            (str,  None, 'Request', 'new location of the torrents content'),
        "peer-limit":          (int,  None, 'Request', 'maximum number of peers'),
        "priority-high":       (list, None, 'Request', 'indices of high-priority file(s)'),
        "priority-low":        (list, None, 'Request', 'indices of low-priority file(s)'),
        "priority-normal":     (list, None, 'Request', 'indices of normal-priority file(s)'),
        "queuePosition":       (int,  None, 'Request', 'position of this torrent in its queue [0...n]'),
        "seedIdleLimit":       (int,  None, 'Request', 'torrent-level number of minutes of seeding inactivity'),
        "seedIdleMode":        (int,  None, 'Request', 'which seeding inactivity to use.  See tr_idlelimit'),
        "seedRatioLimit":      (int, None, 'Request', 'torrent-level seeding ratio'),
        "seedRatioMode":       (int,  None, 'Request', 'which ratio to use.  See tr_ratiolimit'),
        "trackerAdd":          (list, None, 'Request', 'strings of announce URLs to add'),
        "trackerRemove":       (list, None, 'Request', 'ids of trackers to remove'),
        "trackerReplace":      (list, None, 'Request', 'pairs of <trackerId/new announce URLs>'),
        "uploadLimit":         (int,  None, 'Request', 'maximum upload speed (KBps)'),
        "uploadLimited":       (bool, None, 'Request', 'true if "uploadLimit" is honored'),
    },
    'torrent-get': {
        'activityDate':            (int,    None, 'Response', 'Last time of upload or download activity.'),
        'addedDate':               (int,    None, 'Response', 'The date when this torrent was first added.'),
        'announceResponse':        (str,    None, 'Response', 'The announce message from the tracker.'),
        'announceURL':             (str,    None, 'Response', 'Current announce URL.'),
        'bandwidthPriority':       (int,    None, 'Response', 'Bandwidth priority. Low (-1), Normal (0) or High (1).'),
        'comment':                 (str,    None, 'Response', 'Torrent comment.'),
        'corruptEver':             (int,    None, 'Response', 'Number of bytes of corrupt data downloaded.'),
        'creator':                 (str,    None, 'Response', 'Torrent creator.'),
        'dateCreated':             (int,    None, 'Response', 'Torrent creation date.'),
        'desiredAvailable':        (int,    None, 'Response', 'Number of bytes avalable and left to be downloaded.'),
        'doneDate':                (int,    None, 'Response', 'The date when the torrent finished downloading.'),
        'downloadDir':             (str,    None, 'Response', 'The directory path where the torrent is downloaded to.'),
        'downloadedEver':          (int,    None, 'Response', 'Number of bytes of good data downloaded.'),
        'downloaders':             (int,    None, 'Response', 'Number of downloaders.'),
        'downloadLimit':           (int,    None, 'Response', 'Download limit in Kbps.'),
        'downloadLimited':         (bool,   None, 'Response', 'Download limit is enabled'),
        'downloadLimitMode':       (int,    None, 'Response', 'Download limit mode. 0 means global, 1 means signle, 2 unlimited.'),
        'error':                   (int,    None, 'Response', 'Kind of error. 0 means OK, 1 means tracker warning, 2 means tracker error, 3 means local error.'),
        'errorString':             (int,    None, 'Response', 'Error message.'),
        'eta':                     (int,    None, 'Response', 'Estimated number of seconds left when downloading or seeding. -1 means not available and -2 means unknown.'),
        'etaIdle':                 (int,    None, 'Response', 'Estimated number of seconds left until the idle time limit is reached. -1 means not available and -2 means unknown.'),
        'files':                   (list,   None, 'Response', 'Array of file object containing key, bytesCompleted, length and name.'),
        'fileStats':               (list,   None, 'Response', 'Aray of file statistics containing bytesCompleted, wanted and priority.'),
        'hashString':              (str,    None, 'Response', 'Hashstring unique for the torrent even between sessions.'),
        'haveUnchecked':           (int,    None, 'Response', 'Number of bytes of partial pieces.'),
        'haveValid':               (int,    None, 'Response', 'Number of bytes of checksum verified data.'),
        'honorsSessionLimits':     (bool,   None, 'Response', 'True if session upload limits are honored'),
        'id':                      (int,    None, 'Response', 'Session unique torrent id.'),
        'isFinished':              (bool,   None, 'Response', 'True if the torrent is finished. Downloaded and seeded.'),
        'isPrivate':               (bool,   None, 'Response', 'True if the torrent is private.'),
        'isStalled':               (bool,   None, 'Response', 'True if the torrent has stalled (been idle for a long time).'),
        'lastAnnounceTime':        (int,    None, 'Response', 'The time of the last announcement.'),
        'lastScrapeTime':          (int,    None, 'Response', 'The time af the last successful scrape.'),
        'leechers':                (int,    None, 'Response', 'Number of leechers.'),
        'leftUntilDone':           (int,    None, 'Response', 'Number of bytes left until the download is done.'),
        'magnetLink':              (str,    None, 'Response', 'The magnet link for this torrent.'),
        'manualAnnounceTime':      (int,    None, 'Response', 'The time until you manually ask for more peers.'),
        'maxConnectedPeers':       (int,    None, 'Response', 'Maximum of connected peers.'),
        'metadataPercentComplete': (int,    None, 'Response', 'Download progress of metadata. 0.0 to 1.0.'),
        'name':                    (str,    None, 'Response', 'Torrent name.'),
        'nextAnnounceTime':        (int,    None, 'Response', 'Next announce time.'),
        'nextScrapeTime':          (int,    None, 'Response', 'Next scrape time.'),
        'peer-limit':              (int,    None, 'Response', 'Maximum number of peers.'),
        'peers':                   (list,   None, 'Response', 'Array of peer objects.'),
        'peersConnected':          (int,    None, 'Response', 'Number of peers we are connected to.'),
        'peersFrom':               (object, None, 'Response', 'Object containing download peers counts for different peer types.'),
        'peersGettingFromUs':      (int,    None, 'Response', 'Number of peers we are sending data to.'),
        'peersKnown':              (int,    None, 'Response', 'Number of peers that the tracker knows.'),
        'peersSendingToUs':        (int,    None, 'Response', 'Number of peers sending to us'),
        'percentDone':             (int,   None, 'Response', 'Download progress of selected files. 0.0 to 1.0.'),
        'pieces':                  (str,    None, 'Response', 'String with base64 encoded bitfield indicating finished pieces.'),
        'pieceCount':              (int,    None, 'Response', 'Number of pieces.'),
        'pieceSize':               (int,    None, 'Response', 'Number of bytes in a piece.'),
        'priorities':              (list,   None, 'Response', 'Array of file priorities.'),
        'queuePosition':           (int,    None, 'Response', 'The queue position.'),
        'rateDownload':            (int,    None, 'Response', 'Download rate in bps.'),
        'rateUpload':              (int,    None, 'Response', 'Upload rate in bps.'),
        'recheckProgress':         (int,   None, 'Response', 'Progress of recheck. 0.0 to 1.0.'),
        'secondsDownloading':      (int,    None, 'Response', ''),
        'secondsSeeding':          (int,    None, 'Response', ''),
        'scrapeResponse':          (str,    None, 'Response', 'Scrape response message.'),
        'scrapeURL':               (str,    None, 'Response', 'Current scrape URL'),
        'seeders':                 (int,    None, 'Response', 'Number of seeders reported by the tracker.'),
        'seedIdleLimit':           (int,    None, 'Response', 'Idle limit in minutes.'),
        'seedIdleMode':            (int,    None, 'Response', 'Use global (0), torrent (1), or unlimited (2) limit.'),
        'seedRatioLimit':          (int,   None, 'Response', 'Seed ratio limit.'),
        'seedRatioMode':           (int,    None, 'Response', 'Use global (0), torrent (1), or unlimited (2) limit.'),
        'sizeWhenDone':            (int,    None, 'Response', 'Size of the torrent download in bytes.'),
        'startDate':               (int,    None, 'Response', 'The date when the torrent was last started.'),
        'status':                  (int,    None, 'Response', 'Current status, see source'),
        'swarmSpeed':              (int,    None, 'Response', 'Estimated speed in Kbps in the swarm.'),
        'timesCompleted':          (int,    None, 'Response', 'Number of successful downloads reported by the tracker.'),
        'trackers':                (list,   None, 'Response', 'Array of tracker objects.'),
        'trackerStats':            (object, None, 'Response', 'Array of object containing tracker statistics.'),
        'totalSize':               (int,    None, 'Response', 'Total size of the torrent in bytes'),
        'torrentFile':             (str,    None, 'Response', 'Path to .torrent file.'),
        'uploadedEver':            (int,    None, 'Response', 'Number of bytes uploaded, ever.'),
        'uploadLimit':             (int,    None, 'Response', 'Upload limit in Kbps'),
        'uploadLimitMode':         (int,    None, 'Response', 'Upload limit mode. 0 means global, 1 means signle, 2 unlimited.'),
        'uploadLimited':           (bool,   None, 'Response', 'Upload limit enabled.'),
        'uploadRatio':             (int,   None, 'Response', 'Seed ratio.'),
        'wanted':                  (list,   None, 'Response', 'Array of booleans indicated wanted files.'),
        'webseeds':                (list,   None, 'Response', 'Array of webseeds objects'),
        'webseedsSendingToUs':     (int,    None, 'Response', 'Number of webseeds seeding to us.'),
    },
    'torrent-add': {
        "cookies":           (str,  None, 'Request', 'pointer to a string of one or more cookies.'),
        "download-dir":      (str,  None, 'Request', 'path to download the torrent to'),
        "filename":          (str,  None, 'Request', 'filename or URL of the .torrent file'),
        "metainfo":          (str,  None, 'Request', 'base64-encoded .torrent content'),
        "paused":            (bool, None, 'Request', 'if true, dont start the torrent'),
        "peer-limit":        (int,  None, 'Request', 'maximum number of peers'),
        "bandwidthPriority": (int,  None, 'Request', 'torrents bandwidth tr_priority_t'),
        "files-wanted":      (list, None, 'Request', 'indices of file(s) to download'),
        "files-unwanted":    (list, None, 'Request', 'indices of file(s) to not download'),
        "priority-high":     (list, None, 'Request', 'indices of high-priority file(s)'),
        "priority-low":      (list, None, 'Request', 'indices of low-priority file(s)'),
        "priority-normal":   (list, None, 'Request', 'indices of normal-priority file(s)'),
    },
    'torrent-remove': {
        "ids":               (list, None, 'Request', 'torrent list, as described in 3.1'),
        "delete-local-data": (bool, None, 'Request', 'delete local data. (default: false)'),
    },
    'torrent-set-location': {
        "ids":      (list, None, 'Request','torrent list, as described in 3.1'),
        "location": (str,  None, 'Request', 'the new torrent location'),
        "move":     (bool, None, 'Request', 'if true, move from previous location. otherwise, search "location" for files (default: false)'),
    },
    'torrent-rename-path': {
        "ids":  (list, None, 'Request', 'the torrent torrent list, as described in 3.1 (must only be 1 torrent)'),
        "path": (str,  None, 'Request', 'the path to the file or folder that will be renamed'),
        "name": (str,  None, 'Request', 'the file or folders new name')
    },
    'session-set': {
        "alt-speed-down":               ( int  ,  None, 'All',      'max global download speed (KBps)'),
        "alt-speed-enabled":            ( bool ,  None, 'All',      'true means use the alt speeds'),
        "alt-speed-time-begin":         ( int  ,  None, 'All',      'when to turn on alt speeds (units: minutes after midnight)'),
        "alt-speed-time-enabled":       ( bool ,  None, 'All',      'true means the scheduled on/off times are used'),
        "alt-speed-time-end":           ( int  ,  None, 'All',      'when to turn off alt speeds (units: same)'),
        "alt-speed-time-day":           ( int  ,  None, 'All',      'what day(s) to turn on alt speeds (look at tr_sched_day)'),
        "alt-speed-up":                 ( int  ,  None, 'All',      'max global upload speed (KBps)'),
        "blocklist-url":                ( str  ,  None, 'All',      'location of the blocklist to use for "blocklist-update"'),
        "blocklist-enabled":            ( bool ,  None, 'All',      'true means enabled'),
        "blocklist-size":               ( int  ,  None, 'All',      'number of rules in the blocklist'),
        "cache-size-mb":                ( int  ,  None, 'All',      'maximum size of the disk cache (MB)'),
        "config-dir":                   ( str  ,  None, 'Response', 'location of transmissions configuration directory'),
        "download-dir":                 ( str  ,  None, 'All',      'default path to download torrents'),
        "download-queue-size":          ( int  ,  None, 'All',      'max number of torrents to download at once (see download-queue-enabled)'),
        "download-queue-enabled":       ( bool ,  None, 'All',      'if true, limit how many torrents can be downloaded at once'),
        "dht-enabled":                  ( bool ,  None, 'All',      'true means allow dht in public torrents'),
        "encryption":                   ( str  ,  None, 'All',      '"required", "preferred", "tolerated"'),
        "idle-seeding-limit":           ( int  ,  None, 'All',      "torrents we're seeding will be stopped if they're idle for this long"),
        "idle-seeding-limit-enabled":   ( bool ,  None, 'All',      'true if the seeding inactivity limit is honored by default'),
        "incomplete-dir":               ( str  ,  None, 'All',      'path for incomplete torrents, when enabled'),
        "incomplete-dir-enabled":       ( bool ,  None, 'All',      'true means keep torrents in incomplete-dir until done'),
        "lpd-enabled":                  ( bool ,  None, 'All',      'true means allow Local Peer Discovery in public torrents'),
        "peer-limit-global":            ( int  ,  None, 'All',      'maximum global number of peers'),
        "peer-limit-per-torrent":       ( int  ,  None, 'All',      'maximum global number of peers'),
        "pex-enabled":                  ( bool ,  None, 'All',      'true means allow pex in public torrents'),
        "peer-port":                    ( int  ,  None, 'All',      'port number'),
        "peer-port-random-on-start":    ( bool ,  None, 'All',      'true means pick a random peer port on launch'),
        "port-forwarding-enabled":      ( bool ,  None, 'All',      'true means enabled'),
        "queue-stalled-enabled":        ( bool ,  None, 'All',      'whether or not to consider idle torrents as stalled'),
        "queue-stalled-minutes":        ( int  ,  None, 'All',      "torrents that are idle for N minuets aren't counted toward seed-queue-size or download-queue-size"),
        "rename-partial-files":         ( bool ,  None, 'All',      'true means append ".part" to incomplete files'),
        "rpc-version":                  ( int  ,  None, 'Response', 'the current RPC API version'),
        "rpc-version-minimum":          ( int  ,  None, 'Response', 'the minimum RPC API version supported'),
        "script-torrent-done-filename": ( str  ,  None, 'All',      'filename of the script to run'),
        "script-torrent-done-enabled":  ( bool ,  None, 'All',      'whether or not to call the "done" script'),
        "seedRatioLimit":               ( int ,  None, 'All',      'the default seed ratio for torrents to use'),
        "seedRatioLimited":             ( bool ,  None, 'All',      'true if seedRatioLimit is honored by default'),
        "seed-queue-size":              ( int  ,  None, 'All',      'max number of torrents to uploaded at once (see seed-queue-enabled)'),
        "seed-queue-enabled":           ( bool ,  None, 'All',      'if true, limit how many torrents can be uploaded at once'),
        "speed-limit-down":             ( int  ,  None, 'All',      'max global download speed (KBps)'),
        "speed-limit-down-enabled":     ( bool ,  None, 'All',      'true means enabled'),
        "speed-limit-up":               ( int  ,  None, 'All',      'max global upload speed (KBps)'),
        "speed-limit-up-enabled":       ( bool ,  None, 'All',      'true means enabled'),
        "start-added-torrents":         ( bool ,  None, 'All',      'true means added torrents will be started right away'),
        "trash-original-torrent-files": ( bool ,  None, 'All',      'true means the .torrent file of added torrents will be deleted'),
        "units":                        ( object, None, 'All',      'see below'),
        "utp-enabled":                  ( bool ,  None, 'All',      'true means allow utp'),
        "version":                      ( str  ,  None, 'Response', 'long version string "$version ($revision)"')
    },
    'session-stats': {
        "activeTorrentCount": ( int,    None, 'Response', 'Number of active torrents'),
        "downloadSpeed":      ( int,    None, 'Response', 'Download Speed'),
        "pausedTorrentCount": ( int,    None, 'Response', 'Number of paused torrent'),
        "torrentCount":       ( int,    None, 'Response', 'Number of torrents'),
        "uploadSpeed":        ( int,    None, 'Response', 'Upload Speed'),
        "cumulative-stats":   ( object, None, 'Response', ''),
        "current-stats":      ( object, None, 'Response', '')
    },
    "blocklist-update": {
        "blocklist-size": (int, None, 'Response', '')
    },
    "port-test": {
        "port-is-open": (bool, None, 'Response', '')
    },
    "queue-move-top": {
        "ids": (list, None, 'Response', '')
    },
    "queue-move-up": {
        "ids": (list, None, 'Response', '')
    },
    "queue-move-down": {
        "ids": (list, None, 'Response', '')
    },
    "queue-move-bottom": {
        "ids": (list, None, 'Response', '')
    },
    "free-space": {
        "path":       (str, None, 'All', ''),
        "size-bytes": (int, None, 'Response', 'the size, in bytes, of the free space in that directory')
    }
}




import os
from collections import OrderedDict
import json
from urllib.parse import urlsplit

from .server import DEFAULT_ADDR, DEFAULT_PORT, Server
from .exceptions import EmissionBadParamater
from .helpers import *


class Client(object):

    def __init__(self,
                 address=DEFAULT_ADDR,
                 port=DEFAULT_PORT,
                 user=None,
                 password=None,
                 timeout=None):

        self._server = Server(address, port)
        self._tag = 0

    def _update_tag(self):

        self._tag = self._tag + 1

    def emission(self, method, body):

        response = self._server.execute(body)
        if response:
            return json.loads(response.text)
        else:
            return None

    def start(self, ids=None):

        method = 'torrent-start'

        arguments = OrderedDict()
        if ids:
            arguments['ids'] = check_ids(ids)

        payload = OrderedDict()
        payload['method'] = method
        payload['arguments'] = arguments
        payload['tag'] = self._tag

        self._update_tag()

        return self.emission(method, payload)

    def start_now(self, ids=None):

        method = 'torrent-start-now'

        arguments = {}
        if ids:
            arguments['ids'] = check_ids(ids)

        payload = OrderedDict()
        payload['method'] = method
        payload['arguments'] = arguments
        payload['tag'] = self._tag

        self._update_tag()

        return self.emission(method, payload)

    def stop(self, ids=None):

        method = 'torrent-stop'

        arguments = {}
        if ids:
            arguments['ids'] = check_ids(ids)

        payload = OrderedDict()
        payload['method'] = method
        payload['arguments'] = arguments
        payload['tag'] = self._tag

        self._update_tag()

        return self.emission(method, payload)

    def verify(self, ids=None):

        method = 'torrent-verify'

        arguments = {}
        if ids:
            arguments['ids'] = check_ids(ids)

        payload = OrderedDict()
        payload['method'] = method
        payload['arguments'] = arguments
        payload['tag'] = self._tag

        self._update_tag()

        return self.emission(method, payload)

    def reannounce(self, ids=None):
        method = 'torrent-reannounce'

        arguments = {}
        if ids:
            arguments['ids'] = check_ids(ids)

        payload = OrderedDict()
        payload['method'] = method
        payload['arguments'] = arguments
        payload['tag'] = self._tag

        self._update_tag()

        return self.emission(method, payload)

    def list_torrent(self, fields=['id', 'name'], ids=None):

        method = 'torrent-get'

        fields = normalize_fields_arguments(method, fields)

        arguments = {}
        arguments['fields'] = fields
        if ids:
            arguments['ids'] = check_ids(ids)

        payload = OrderedDict()
        payload['method'] = method
        payload['arguments'] = arguments
        payload['tag'] = self._tag

        self._update_tag()

        response = self.emission(method, payload)
        return response['arguments']

    def add_torrent(self, torrent, **kwargs):

        method = 'torrent-add'
        if not torrent:
            raise EmissionBadParamater('torrent should not be empty')

        args = normalize_arguments(method, kwargs)

        torrent_data = ""
        torrent_file = ""

        t_url = urlsplit(torrent)

        if t_url[0] in ('http', 'https'):
            # Problems with https
            # so download the torrent file instead to send to transmission
            _url = to_http(torrent)
            _r = self._server.download(_url)
            torrent_data = decoder(_r.content)
        elif t_url[0] == 'file':
            if not os.path.exists(t_url[2]):
                raise EmissionBadParamater("'{0}' does not exists !!".format(t_url[2]))
            torrent_file = t_url[2]
        else:
            raise EmissionBadParamater('bad http scheme')

        arguments = {k: v for k, v in args}
        if torrent_data:
            arguments['metainfo'] = torrent_data
        if torrent_file:
            arguments['filename'] = torrent_file

        payload = OrderedDict()
        payload['method'] = method
        payload['arguments'] = arguments
        payload['tag'] = self._tag

        self._update_tag()

        response = self.emission(method, payload)
        return response['arguments']

    def delete_torrent(self, ids, **kwargs):

        method = 'torrent-remove'
        if not ids:
            raise EmissionBadParamater('ids should not be empty')

        args = normalize_arguments(method, kwargs)

        arguments = {k: v for k, v in args}
        arguments['ids'] = ids

        payload = OrderedDict()
        payload['method'] = method
        payload['arguments'] = arguments
        payload['tag'] = self._tag

        self._update_tag()

        response = self.emission(method, payload)
        return response['arguments']

    def queue(self, where, ids):

        def get_method(where):

            if where == 'top':
                return 'queue-move-top'
            elif where == 'up':
                return 'queue-move-up'
            elif where == 'down':
                return 'queue-move-down'
            elif where == 'bottom':
                return 'queue-move-bottom'

        if not where or not ids:
            raise EmissionBadParamater('where or ids should not be empty')

        method = get_method(where)

        arguments = {}
        arguments['ids'] = check_ids(ids)

        payload = OrderedDict()
        payload['method'] = method
        payload['arguments'] = arguments
        payload['tag'] = self._tag

        self._update_tag()

        response = self.emission(method, payload)
        return response['arguments']

    # Session

    def get_session(self):

        method = 'session-get'

        payload = OrderedDict()
        payload['method'] = method
        payload['tag'] = self._tag

        self._update_tag()

        response = self.emission(method, payload)
        return response['arguments']

    def set_session(self, **kwargs):

        method = 'session-set'

        if not kwargs:
            raise EmissionBadParamater('should have at least one argument')

        args = normalize_arguments(method, kwargs)

        arguments = {k: v for k, v in args}

        payload = OrderedDict()
        payload['method'] = method
        payload['arguments'] = arguments
        payload['tag'] = self._tag

        self._update_tag()

        response = self.emission(method, payload)
        print(response)

    def stat_session(self):

        method = 'session-stats'

        payload = OrderedDict()
        payload['method'] = method
        payload['tag'] = self._tag

        self._update_tag()

        response = self.emission(method, payload)
        return response['arguments']

    def close_session(self):

        method = 'session-close'

        payload = OrderedDict()
        payload['method'] = method
        payload['tag'] = self._tag

        self._update_tag()

        response = self.emission(method, payload)
        return response

    def blocklist(self):

        method = 'blocklist-update'

        payload = OrderedDict()
        payload['method'] = method
        payload['tag'] = self._tag

        self._update_tag()

        response = self.emission(method, payload)
        return response['arguments']

    def port_test(self):

        method = 'port-test'

        payload = OrderedDict()
        payload['method'] = method
        payload['tag'] = self._tag

        self._update_tag()

        response = self.emission(method, payload)
        return response['arguments']

    def free_space(self, path):

        method = 'free-space'

        if not path:
            raise EmissionBadParamater("'path' should not be empty")

        arguments = {}
        arguments['path'] = path

        payload = OrderedDict()
        payload['method'] = method
        payload['arguments'] = arguments
        payload['tag'] = self._tag

        self._update_tag()

        response = self.emission(method, payload)
        return response['arguments']



version = '0.3.0'

name = 'emissionrpc'

description = 'Emission, Another Wrapper JSON-RPC to Transmission'

url = 'https://github.com/Germione/emissionrpc'

author = 'Jerome Pradier'
author_email = 'jerome.pradier@gmail.com'

license = 'MIT'




import json
import requests
from requests.exceptions import RequestException, HTTPError

from .exceptions import EmissionLibError, EmissionBadParamater

DEFAULT_ADDR = 'localhost'
DEFAULT_PORT = '9091'
DEFAULT_TIMEOUT = '30'
ENDPOINT = '/transmission/rpc'


class Server(object):

    ENDPOINT = '/transmission/rpc'

    def __init__(self, address, port, timeout=None):
        self._session = requests.Session()
        self._url = 'http://{0}:{1}{2}'.format(address, port, self.ENDPOINT)
        self._token = 'empty'
        self.handle_csrf()

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, val):
        try:
            address, port = val
        except ValueError:
            raise EmissionBadParamater("pass an iterable with address and port")
        else:
            """ This will run only if no exception was raised """
            self._url = 'http://{0}:{1}{2}'.format(address, port, self.ENDPOINT)

    def handle_csrf(self):
        try:
            r = self._session.get(self._url)
            r.raise_for_status()
        except HTTPError as e:
            if e.response.status_code == 409:
                self._token = e.response.headers['X-Transmission-Session-Id']
            else:
                raise EmissionLibError("something went wrong with request", e)
        except RequestException as e:
            raise EmissionLibError("something went wrong with request", e)

    def download(self, url):
        try:
            r = self._session.get(url)
            r.raise_for_status()
            return r
        except (RequestException, HTTPError) as e:
            raise EmissionLibError("something went wrong with request", e)

    def execute(self, payload):
        if not self._token:
            raise EmissionBadParamater('token should not be empty')

        token = {'X-Transmission-Session-Id': self._token}

        try:
            r = self._session.post(self._url,
                                   headers=token,
                                   data=json.dumps(payload))
            r.raise_for_status()
            return r
        except (RequestException, HTTPError) as e:
            raise EmissionLibError("something went wrong with request", e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

import keyring
import keyring.util.platform_


# Needed for the following functions
here = os.path.abspath(os.path.dirname(__file__))


# Load the package's __version__.py module as a dictionary.
about = {}
with open(os.path.join(here, 'emissionrpc', '__version__.py')) as f:
    exec(f.read(), about)


# Get the long description from the README file
def get_long_description():

    with open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()

    return long_description


class SecureUploadCommand(Command):
    """
    python setup.py upload --repository pypi
    python setup.py upload --repository testpypi
    """

    description = 'Build and publish the package.'
    user_options = [('repository=', 'r', 'repository')]
    servers = ['pypi', 'testpypi']

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        self.repository = None

    def finalize_options(self):
        if self.repository and self.repository in self.servers:
            self.server = self.repository
        else:
            raise Exception("Missing repository argument [ 'pypi' or 'testpypi' ]")

    def get_credentials(self):

        self.status('Getting credentials')

        # Getting the keyring data from the filesystem
        keyring.get_keyring()

        # Then get the password
        self.username = "Germione"
        self.password = keyring.get_password("pypi", self.username)

    def build_sdist(self):

        self.status('Building Source distribution...')
        os.system('{0} setup.py sdist'.format(sys.executable))

    def build_wheel(self):

        self.status('Building Wheel (universal) distribution...')
        os.system('{0} setup.py sdist bdist_wheel'.format(sys.executable))

    def twine(self):
        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload -r {0} -u {1} -p {2} dist/*'.format(self.server, self.username, self.password))

    def run(self):
        try:
            self.status('Removing previous builds...')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.get_credentials()

        self.build_sdist()

        self.build_wheel()

        self.twine()

        sys.exit()


setup(

    # Name of the package
    name=about['name'],

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=about['version'],

    description=about['description'],
    long_description=get_long_description(),

    # The project's main homepage.
    url=about['url'],

    # Author details
    author=about['author'],
    author_email=about['author_email'],

    # Choose your license
    # Don't forget to the license file
    license=about['license'],

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],

    # What does your project relate to?
    keywords=' torrent json rpc',

    # Adding Non-Code Files :
    # In order for these files to be copied at install time to the package’s folder inside site-packages,
    # you’ll need to supply :
    include_package_data=True,

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['twine', 'keyring'],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    # See https://setuptools.readthedocs.io/en/latest/setuptools.html#automatic-script-creation
    #entry_points=dict(console_scripts=['emission = emission:cli']),

    cmdclass={
        'upload': SecureUploadCommand,
    },
)




"""
emissionrpc Library
~~~~~~~~~~~~~~~~~~~~~
Wrapper JSON-RPC for transmission.

usage:
   >>> import emission
   >>> c = emissionrpc.Client()
   >>> c.list_torrent()
"""

from .helpers import check_ids
from .exceptions import EmissionException, EmissionBadParamater, EmissionLibError
from .server import Server
from .emitter import Client





class EmissionException(Exception):
    """
    Basic exception for errors raised by EmissionRpc
    """


class EmissionBadParamater(EmissionException):
    """
    Options does not exists
    """
    pass


class EmissionLibError(Exception):
    """
    Generic exception for wrapping others exceptions from any library
    """
    def __init__(self, msg, original_exception):
        super(EmissionLibError, self).__init__("{0}: {1}".format(msg, original_exception))
        self.original_exception = original_exception

